import { createConnection } from "typeorm"
import { User } from "../../Auth/User.entity"
import { Event, EventInformation, EventOccurrence } from "../Event.entity"
import testConfig from "../../__tests__/testORMConfig"

let owner: User = null

let connection
describe("Recurring events", () => {
  beforeAll(async () => {
    connection = await createConnection({
      ...testConfig,
    })
    owner = new User()
    owner.username = "onetimeguy"
    owner.email = "one@time.com"
    owner.name = "One Time"
    await owner.save()
    return connection
  })
  afterAll(async () => {
    await connection.close()
  })

  it("Event with no recurrence has one occurrence", async done => {
    const event = new Event()
    const info = new EventInformation()

    info.title = "Test one time event"
    info.description = "This is a test event that has no repetition rules"
    event.owner = Promise.resolve(owner)
    event.time = {
      timeZone: "Europe/Madrid",
      start: "2019-02-01T10:00Z",
      end: "2019-02-01T11:00Z",
    }
    event.occurrences = Promise.resolve(event.getOccurrences())
    await event.save()

    const occurences = await EventOccurrence.find({
      where: { eventId: event.id },
    })
    expect(occurences).toHaveLength(1)
    expect((await occurences[0].event).time.timeZone).toEqual(
      event.time.timeZone
    )
    done()
  })

  it("Event happening once a week", async () => {
    const event = new Event()
    const info = new EventInformation()
    info.language = "en"
    info.title = "Weekly testing event"
    info.description = "This event takes place every monday at 2pm"

    event.infos = Promise.resolve([info])
    event.owner = Promise.resolve(owner)

    // from 2019-03-04 to 2019-03-25 every monday
    //      March 2019
    // Su Mo Tu We Th Fr Sa
    //                1  2
    // 3  4  5  6  7  8  9    <- 2019-03-04
    // 10 11 12 13 14 15 16   <- 2019-03-11
    // 17 18 19 20 21 22 23   <- 2019-03-18
    // 24 25 26 27 28 29 30   <- 2019-03-25
    // 31

    event.time = {
      timeZone: "Europe/Madrid",
      start: "2019-03-04T14:00",
      end: "2019-03-04T15:00",
      recurrence:
        "DTSTART:20190304T140000Z\nRRULE:FREQ=WEEKLY;BYDAY=MO;INTERVAL=1;UNTIL=20190325T230000Z", // until 15/04/2019
    }

    event.occurrences = Promise.resolve(event.getOccurrences())
    expect(await event.occurrences).toHaveLength(4)
  })
})
